# Soya 3D
# Copyright (C) 2001-2002 Jean-Baptiste LAMY -- jiba@tuxfamily.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import _soya, soya
    
class Widget(object):
  def __init__(self, master = None):
    # these attributes are named (left,top) and not (x,y) to avoid confusion with
    # the (x,y,z) attributes of 3D objects
    self.left = 0
    self.top  = 0
    self.width  = 0
    self.height = 0
    # this attribute is not called parent for avoiding confusion with 3D objects parent
    # attribute and also because Camera is a 3D object and a Widget
    if master: master.add(self)
    else: self.master = None
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    # this method must be rewritten for each widget instance ;)
    pass
  def begin_round(self): pass
  def advance_time(self, proportion): pass
  def end_round(self): pass
  
class Group(Widget):
  def __init__(self, master = None):
    Widget.__init__(self, master)
    self.children = []
  def render(self):
    for widget in self.children: widget.render()
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left   = parent_left
    self.top    = parent_top
    self.width  = parent_width
    self.height = parent_height
    for widget in self.children: widget.resize(parent_left, parent_top, parent_width, parent_height)
  def add(self, widget):
    self.children.append(widget)
    widget.resize(self.left, self.top, self.width, self.height)
    widget.master = self
  def insert(self, index, widget):
    self.children.insert(index, widget)
    widget.resize(self.left, self.top, self.width, self.height)
    widget.master = self
  def remove(self, widget):
    self.children.remove(widget)
    widget.master = None
  def begin_round(self):
    for widget in self.children: widget.begin_round()      
  def advance_time(self, proportion):
    for widget in self.children: widget.advance_time(proportion)      
  def end_round(self):
    for widget in self.children: widget.end_round()      
  


#default_font = _soya.font_raster_load(os.path.join(os.path.dirname(soya.__file__), "data", "font_8"))
#default_font = _soya.font_texture_load(os.path.join(os.path.dirname(soya.__file__), "data", "font_time_22"))
default_font = soya.load_font(os.path.join(os.path.dirname(soya.__file__), "data", "font_time_22"))


class Label(Widget):
  def __init__(self, master = None, text = "", align = 0, color1 = (1.0, 1.0, 1.0, 1.0), color2 = None, font = default_font):
    Widget.__init__(self, master)
    self.text   = text
    self.color1 = color1
    self.color2 = color2 or color1
    self.font   = font
    self.scale  = (1.0, 1.0)
    # align: 0 left, 1 center
    self.align  = align
  def render(self):
    if self.text:
      self.font.draw_area(self.text, self.left, self.top, self.width, self.height, self.align, self.scale, self.color1, self.color2)

class OptimizedLabel(Widget):
  def __init__(self, master = None, text = "", align = 0, color = (1.0, 1.0, 1.0, 1.0), font = default_font):
    self._text = text
    self._color = color
    # font must be a TextureFont or it crashes (you MUST NOT use a RasterFont)
    self.font = font
    # align: 0 left, 1 center
    self.align = align
    self.call_list = -1
    self.valid = 0
    Widget.__init__(self, master)
  def __del__(self):
    _soya.glDeleteList(self.call_list)
  def render(self):
    if self._text:
      if self.valid == 0:
        if self.call_list == -1: self.call_list = _soya.glGenList()
        _soya.glNewList(self.call_list)
        _soya.glColor4f(*self._color)
        self.height = self.font.draw_area(self._text, 0, 0, self.width, -1, self.align)
        _soya.glEndList()
        self.valid = 1
      self.font.set_position(self.left, self.top)
      _soya.glCallList(self.call_list)
      self.font.set_position()
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.width = parent_width
    self.valid = 0
    
  def get_text(self): return self._text
  def set_text(self, text):
    self._text = text
    self.valid = 0
  text = property(get_text, set_text)
  
  def get_color(self): return self._color
  def set_color(self, color):
    self._color = color
    self.valid = 0
  color = property(get_color, set_color)

class Image(Widget):
  def __init__(self, master = None, material = None, color = (1.0, 1.0, 1.0, 1.0)):
    Widget.__init__(self, master)
    self.material = material
    self.tex_top    = 0.0
    self.tex_left   = 0.0
    self.tex_bottom = 1.0
    self.tex_right  = 1.0
    self.tex_mode = 0
    self.color = color
    # mode 0 : normal
    # mode 1 : texcoords follow the height and width of the widget
  def render(self):
    if self.material:
      self.material.activate()
      if self.material.is_alpha(): _soya.glEnable(_soya.GL_BLEND)
    else: _soya.glDisable(_soya.GL_TEXTURE_2D)
    _soya.glColor4f(*self.color)
    _soya.glBegin(_soya.GL_QUADS)
    _soya.glTexCoord2f(self.tex_left, self.tex_top)
    _soya.glVertex3f(float(self.left),              float(self.top),               0.0)
    _soya.glTexCoord2f(self.tex_left, self.tex_bottom)
    _soya.glVertex3f(float(self.left),              float(self.top + self.height), 0.0)
    _soya.glTexCoord2f(self.tex_right, self.tex_bottom)
    _soya.glVertex3f(float(self.left + self.width), float(self.top + self.height), 0.0)
    _soya.glTexCoord2f(self.tex_right, self.tex_top)
    _soya.glVertex3f(float(self.left + self.width), float(self.top),               0.0)
    _soya.glEnd()
    if self.material:
      self.material.inactivate()
      if self.material.is_alpha(): _soya.glDisable(_soya.GL_BLEND)
    else: _soya.glEnable(_soya.GL_TEXTURE_2D)
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    if hasattr(self, "percents"):
      self.left   = self.percents[0] * parent_left
      self.top    = self.percents[1] * parent_top
      self.width  = self.percents[2] * parent_width
      self.height = self.percents[3] * parent_height
      if self.material and self.tex_mode == 1:
        self.tex_right  = float(self.width)  / self.material.width  + self.tex_left
        self.tex_bottom = float(self.height) / self.material.height + self.tex_top


class Choice:
  def __init__(self, label = "", action = None, value = None, range = None, incr = None):
    self.label  = label
    self.action = action
    self.value  = value
    self.range  = range
    self.incr   = incr
  def get_label(self):
    if (self.value != None):
      return "%s - %s" % (self.label, self.value)
    else: return self.label
  def key_down(self, key_id):
    if (key_id == soya.K_RETURN or key_id == soya.K_KP_ENTER or key_id == soya.K_SPACE):
      if (self.action): self.action()
    elif (key_id == soya.K_RIGHT or key_id == soya.K_PLUS or key_id == soya.K_KP_PLUS):
      if (self.value != None):
        if (self.incr):
          self.value = self.value + self.incr
          if (self.range):
            if (self.value < self.range[0]): self.value = self.range[0]
            if (self.value > self.range[1]): self.value = self.range[1]
        else:
          nb = len(self.range)
          i = 0
          while (i < nb):
            if (self.range[i] == self.value):
              i = i + 1
              if (i >= nb): i = 0
              self.value = self.range[i]
              break
            i = i + 1
    elif (key_id == soya.K_LEFT or key_id == soya.K_MINUS or key_id == soya.K_KP_MINUS):
      if (self.value):
        if (self.incr):
          self.value = self.value - self.incr
          if (self.range):
            if (self.value < self.range[0]): self.value = self.range[0]
            if (self.value > self.range[1]): self.value = self.range[1]
        else:
          nb = len(self.range)
          i = 0
          while (i < nb):
            if (self.range[i] == self.value):
              i = i - 1
              if (i < 0): i = nb - 1
              self.value = self.range[i]
              break
            i = i + 1    

class ChoiceList(Widget):
  def __init__(self, master = None, choices = [], font = default_font, color = (1.0, 0.5, 0.5, 1.0), highlight = (1.0, 1.0, 0.0, 1.0)):
    Widget.__init__(self)
    self.choices = choices
    self.font = font
    self.color = color
    self.highlight = highlight
    self.selected = 0
    if master: master.add(self)
    choice_list = self

    import py2play.idler
    class Idler(py2play.idler.Idler):
      def begin_round(self):
        py2play.idler.Idler.begin_round(self)
        for event in soya.process_event():
          if (event[0] == soya.KEYDOWN):
            choice_list.key_down(event[1])
          elif (event[0] == soya.MOUSEMOTION):
            choice_list.mouse_move(event[1], event[2])
          elif (event[0] == soya.MOUSEBUTTONDOWN):
            choice_list.mouse_click()
      def advance_time(self, proportion):
        pass
      def render(self):
        soya.render()

    self.idler = Idler()

  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left   = parent_left
    self.top    = parent_top
    self.width  = parent_width
    self.height = parent_height
  def key_down(self, key_id):
    if key_id == soya.K_DOWN:
      self.selected = self.selected + 1
      if (self.selected >= len(self.choices)): self.selected = 0
    elif key_id == soya.K_UP:
      self.selected = self.selected - 1
      if (self.selected < 0): self.selected = len(self.choices) - 1
    else: self.choices[self.selected].key_down(key_id)
  def mouse_move(self, x, y):
    if (x >= self.left and x <= self.left + self.width):
      i = 0
      nb = len(self.choices)
      h1 = int (self.height / nb)
      h2 = int (h1 * 0.5)
      t = int (self.top + h1 * 0.5)
      while (i < nb):
        if (y >= t - h2 and y <= t + h2):
          self.selected = i
          break
        t = t + h1
        i = i + 1
  def mouse_click(self):
    self.choices[self.selected].key_down(soya.K_RETURN)
  def render(self):
    nb = len (self.choices)
    hi = self.height / nb
    h = int (self.top + hi * 0.5)
    i = 0
    while (i < nb):
      if (i == self.selected): _soya.glColor4f(*self.highlight)
      else: _soya.glColor4f(*self.color)
      self.font.draw_area(self.choices[i].get_label(), self.left, h, self.width, h + hi, 1)
      h = h + hi
      i = i + 1


class FPSLabel(Widget):
  """FPSLabel

A label that shows the FPS.
It works ONLY with soya.idler !!!"""
  
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.x = parent_left + parent_width  - 155
    self.y = parent_top  + parent_height - 37
    
  def render(self):
    import soya.idler as idler
    
    _soya.glColor4f(1.0, 1.0, 1.0, 1.0)
    
    if idler.IDLER:
      default_font.draw_area("%.1f FPS" % idler.IDLER.fps, self.x, self.y, 150, 30, 1)
    else:
      default_font.draw_area("??? FPS", self.x, self.y, 150, 30, 1)



